package it.itsvil.macs.backend.client;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.Invocation;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MultivaluedHashMap;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;

import it.itsvil.macs.backend.dto.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;

import it.itsvil.macs.backend.util.MacsEnum;
import it.itsvil.macs.backend.util.MacsTokenUtil;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Service(value = "PrenotazioneClientImpl")
public class PrenotazioneClientImpl implements PrenotazioneClient {

	@Value(value = "${macs.prenotazione.field.filled}")
	public String FILLED;

	@Autowired
	private Environment env;
	
	@Autowired
	private MacsTokenUtil macsTokenUtil;

	@Override
	public ListaRisorseResponseDTO listaRisorse(CoreRequestDTO request) {

		ListaRisorseResponseDTO response = null;

		log.info("START");

		try {

			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {} ", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		try {

			String output = null;

			response = new ListaRisorseResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_RISORSA_BASE.getCodice(),
					StringUtils.EMPTY);
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url);
			
			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.get();

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("Failed HTTP error code "
						+ resp.getStatus());
			}
			
			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			ContentListaRisorseDTO contentResponse = new Gson().fromJson(output, ContentListaRisorseDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public InfoRisorseResponseDTO infoRisorse(InfoRisorseRequestDTO request) {

		log.info("START");
		
		ObjectMapper mapper = new ObjectMapper();
		
		InfoRisorseResponseDTO response = new InfoRisorseResponseDTO();
		String output = null;

		try {
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
			
			String url = getUrl(
					MacsEnum.MACS_CLIENT_RISORSA_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_RISORSA.getCodice());
			log.info("URL: {}", url);
			
			Client client = ClientBuilder.newClient();
			
			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_RISORSA_ID_RISORSA.getCodice()), request.getIdRisorse());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.get();
			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);
			
			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.infoRisorse.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.infoRisorse. Failed HTTP error code."
						+ resp.getStatus());
			}
			
			RisorseDTO contentResponse = new Gson().fromJson(output, RisorseDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		} catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}
		
		log.info("END");
		return response;
	}

	@Override
	public PeriodiDisponibiliResponseDTO periodiDisponibili(PeriodiDisponibiliRequestDTO request) {

		log.info("START");

		try {

			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		PeriodiDisponibiliResponseDTO response = null;

		try {

			String output = null;

			response = new PeriodiDisponibiliResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_RISORSA_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_RISORSA_PERIODI.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_RISORSA_ID_RISORSA.getCodice()), request.getIdRisorsa())
					.queryParam(env.getProperty(MacsEnum.MACS_CLIENT_DA_QUERY_PARAM.getCodice()), request.getDa())
					.queryParam(env.getProperty(MacsEnum.MACS_CLIENT_A_QUERY_PARAM.getCodice()), request.getA());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.get();

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.periodiDisponibili.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.periodiDisponibili. Failed HTTP error code . "
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			ContentPeriodiDisponibiliDTO contentResponse = new Gson().fromJson(output, ContentPeriodiDisponibiliDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public DownloadDocumentoResponseDTO downloadDocumento(DownloadDocumentoRequestDTO request) {
		log.info("START");

		try {

			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		DownloadDocumentoResponseDTO response = null;

		try {

			String output = null;
			response = new DownloadDocumentoResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_DOCUMENTI.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_PRENOTAZIONE_ID_DOCUMENTO.getCodice()), request.getIdDocumento());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.get();
			
			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);
			
			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.downloadDocumento.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.downloadDocumento. Failed HTTP error code . "
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}

			ContentDownloadDocumentoDTO contentResponse = new Gson().fromJson(output, ContentDownloadDocumentoDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public InitPrenotazioneResponseDTO initPrenotazione(InitPrenotazioneRequestDTO request) {
		log.info("START");

		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		InitPrenotazioneResponseDTO response = null;

		try {

			String output = null;

			response = new InitPrenotazioneResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_RISORSA_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_RISORSA_PRENOTAZIONI.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			log.info("URL: {}", url);

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_RISORSA_ID_RISORSA.getCodice()), request.getIdRisorsa());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			InitPrenotazioneCoreRequestDTO requestCore = new InitPrenotazioneCoreRequestDTO();
			requestCore.setOpenApi(request.getOpenApi());
			requestCore.setSlots(request.getSlots());
			if(null != request.getFromPrtTimeIntervalInterface() 
					&& !request.getFromPrtTimeIntervalInterface().isEmpty()) {
				requestCore.setFromPrtTimeIntervalInterface(request.getFromPrtTimeIntervalInterface());
			}
//			else {
//				requestCore.setFromPrtTimeIntervalInterface(new ArrayList<PrtTimeIntervalInterfaceDTO>());
//			}
			
			
			Response resp = invocationBuilder.post(Entity.entity(requestCore, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.initPrenotazione.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.info("RESPONSE: {}", resp.readEntity(String.class));
				
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.initPrenotazione. Failed HTTP error code . "
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			ContentInitPrenotazioneDTO contentResponse = new Gson().fromJson(output, ContentInitPrenotazioneDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public AnnullaPrenotazioneResponseDTO annullaPrenotazione(AnnullaPrenotazioneRequestDTO request) {
		log.info("START");

		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		AnnullaPrenotazioneResponseDTO response = null;

		try {

			String output = null;

			response = new AnnullaPrenotazioneResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_ANNULLA.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_PRENOTAZIONE_TOKEN.getCodice()), request.getTokenPrenotazione());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
					
			Response resp = invocationBuilder.put(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.annullaPrenotazione.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.annullaPrenotazione. Failed HTTP error code . "
						+ resp.getStatus());
			}
			
			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			AnnullaPrenotazioneContentDTO contentResponse = new Gson().fromJson(output, AnnullaPrenotazioneContentDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			//response = gson.fromJson(output, AnnullaPrenotazioneResponseDTO.class);

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public ConfermaPrenotazioneResponseDTO confermaPrenotazione(ConfermaPrenotazioneRequestDTO request) {
		log.info("START");

		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		ConfermaPrenotazioneResponseDTO response = null;

		try {

			String output = null;

			response = new ConfermaPrenotazioneResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_CONFERMA.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_PRENOTAZIONE_TOKEN.getCodice()), request.getTokenPrenotazione());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			ConfermaPrenotazioneCoreRequestDTO prenotazioneRequest = new ConfermaPrenotazioneCoreRequestDTO();
			prenotazioneRequest.setAll(request);
			
			Response resp = invocationBuilder.put(Entity.entity(prenotazioneRequest, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.confermaPrenotazione.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.confermaPrenotazione. Failed HTTP error code . "
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			ConfermaPrenotazioneContentDTO contentResponse = new Gson().fromJson(output, ConfermaPrenotazioneContentDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public DisdiciPrenotazioneResponseDTO disdiciPrenotazione(DisdiciPrenotazioneRequestDTO request) {
		log.info("START");

		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {} ", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		DisdiciPrenotazioneResponseDTO response = null;

		try {

			String output = null;

			response = new DisdiciPrenotazioneResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_BASE.getCodice(),
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_DISDICI.getCodice());
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url)
					.resolveTemplate(env.getProperty(MacsEnum.MACS_CLIENT_PRENOTAZIONE_ID_PRENOTAZIONE.getCodice()), request.getIdPrenotazione());

			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.put(Entity.entity(request, MediaType.APPLICATION_JSON_VALUE));

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.disdiciPrenotazione.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.disdiciPrenotazione. Failed HTTP error code . "
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			DisdiciPrenotazioneContentDTO contentResponse = new Gson().fromJson(output, DisdiciPrenotazioneContentDTO.class);
			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);

		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}

	@Override
	public ListaPrenotazioneResponseDTO listaPrenotazioni(CoreRequestDTO request) {
		log.info("START");

		try {
			ObjectMapper mapper = new ObjectMapper();
			String jsonString = mapper.writeValueAsString(request);
			log.info("REQUEST: {}", jsonString);
		} catch (JsonProcessingException e) {
			log.error("ERRORE: {}", e.getMessage());
		}

		ListaPrenotazioneResponseDTO response = null;

		try {

			String output = null;

			response = new ListaPrenotazioneResponseDTO();

			String url = getUrl(
					MacsEnum.MACS_CLIENT_PRENOTAZIONE_BASE.getCodice(),
					StringUtils.EMPTY);
			log.info("URL: {}", url);

			Client client = ClientBuilder.newClient();

			WebTarget webTarget = client.target(url);
			
			MultivaluedMap<String, Object> headers = new MultivaluedHashMap<String, Object>();
			headers.add("cache-control", "no-cache");
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_FRUITORE_HEADER.getCodice()), request.getCfFruitore());
			headers.add(env.getProperty(MacsEnum.MACS_CLIENT_CF_PRESENTANTE_HEADER.getCodice()), request.getCfUser());
			
			setToken(headers);
			
			Invocation.Builder invocationBuilder = webTarget
					.request()
					.headers(headers);
			
			Response resp = invocationBuilder.get();

			log.info("SERVIZIO INVOCATO");
			
			output = resp.readEntity(String.class);
			log.info("RESPONSE FROM CLIENT: {}", output);

			if(resp.getStatus() == 401){
				log.error("ERRORE: 401 unauthorized");
				throw new Exception("PrenotazioneClientImpl.listaPrenotazioni.Client internal unauthorized");
			}

			if (resp.getStatus() < 200 || resp.getStatus() >= 300) {
				log.error("ERRORE: Failed HTTP error code");
				throw new Exception("PrenotazioneClientImpl.listaPrenotazioni. Failed HTTP error code."
						+ resp.getStatus());
			}

			try {
				log.info("RESPONSE: {}", output);
			}catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			
			ListaPrenotazioneContentDTO contentResponse = new Gson().fromJson(output, ListaPrenotazioneContentDTO.class);
			PrenotazioneByIdRequestDTO prenotazione = new PrenotazioneByIdRequestDTO();
			try {
				prenotazione = (PrenotazioneByIdRequestDTO) request;
			} catch (Exception e) {
				log.error("ERRORE: {}", e.getMessage());
			}
			for (PrenotazioneAppuntamentoDTO prenotazioneAppuntamento : contentResponse.getPrenotazioni()) {
				if (!prenotazioneAppuntamento.getB64PdfPrenotazione().equals(null)) {
					//if (prenotazione == null || prenotazione.getIdPrenotazione() <= 0 || prenotazione.getIdPrenotazione() != prenotazioneAppuntamento.getIdPrenotazione()) {
						prenotazioneAppuntamento.setB64PdfPrenotazione(FILLED);
					//}
				}
			}

			response.setContent(contentResponse);
			response.setSuccess(true);
			response.setError(null);
			
		}catch (Exception e) {
			log.error("ERRORE: {}", e.getMessage());
			return null;
		}

		log.info("END");
		return response;
	}
	
	/**
	 * 
	 * @param basePathProperty
	 * @param methodProperty
	 * @param param
	 * @return
	 * @throws Exception
	 */
	private String getUrl(String basePathProperty, String methodProperty) throws Exception {
		log.info("START");
		
		String isMockEnable = env.getProperty(MacsEnum.MACS_CLIENT_MOCK_ENABLE.getCodice());
		if (StringUtils.isBlank(isMockEnable))
			throw new Exception("Errore nel recupero della property " + MacsEnum.MACS_CLIENT_MOCK_ENABLE.getCodice());
		
		String basePathUrl = env.getProperty(basePathProperty);
		if (StringUtils.isBlank(basePathUrl))
			throw new Exception("Errore nel recupero della property " + basePathProperty);
		
		String methodUrl = StringUtils.EMPTY;
		if(StringUtils.isNotBlank(methodProperty)) {
			methodUrl = env.getProperty(methodProperty);
			if (StringUtils.isBlank(methodUrl) && StringUtils.isNotBlank(methodProperty))
				throw new Exception("Errore nel recupero della property " + methodProperty);
		}
		
		String initUrl = new String();
		
		StringBuilder completeUrl = new StringBuilder();
		
		//Caso in cui bisogna richiamare i mock
		if(Boolean.TRUE.toString().equalsIgnoreCase(isMockEnable)){
			log.info("MOCK ENABLE TRUE");
			
			initUrl = env.getProperty(MacsEnum.MACS_CLIENT_BASE_MOCK_URL.getCodice());
			if (StringUtils.isBlank(initUrl))
				throw new Exception("Errore nel recupero della property " + MacsEnum.MACS_CLIENT_BASE_MOCK_URL.getCodice());
			
		}
		//Caso in cui bisogna richiamare il client
		else if(Boolean.FALSE.toString().equalsIgnoreCase(isMockEnable)) {
			log.info("MOCK ENABLE FALSE");
			
			initUrl = env.getProperty(MacsEnum.MACS_CLIENT_BASE_URL.getCodice());
			if (StringUtils.isBlank(initUrl))
				throw new Exception("Errore nel recupero della property " + MacsEnum.MACS_CLIENT_BASE_URL.getCodice());
			
		}
		//Caso in cui il valore non e' settato correttamente
		else {
			log.info("MOCK ENABLE NOT VALID");
			throw new Exception("Valore della property " + MacsEnum.MACS_CLIENT_MOCK_ENABLE.getCodice() + " non valido");
		}
		
		completeUrl
			.append(initUrl)
			.append(basePathUrl)
			.append(methodUrl);
		
		log.info("END");
		return completeUrl.toString();
	}
	
	private void setToken(MultivaluedMap<String, Object> headers) {
		String isTokenEnable = env.getProperty(MacsEnum.MACS_TOKEN_ENABLE.getCodice());
		
		if(Boolean.TRUE.toString().equalsIgnoreCase(isTokenEnable)) {
			log.info("TOKEN ENABLE TRUE");
			
			StringBuilder token = new StringBuilder()
					.append(env.getProperty(MacsEnum.MACS_TOKEN_REQUEST_HEADER_BEARER.getCodice()))
					.append(macsTokenUtil.generateToken());
			
			headers.add(env.getProperty(MacsEnum.MACS_TOKEN_REQUEST_HEADER_AUTHORIZATION.getCodice()), token.toString());
			
			return;
		}else if (Boolean.FALSE.toString().equalsIgnoreCase(isTokenEnable)) {
        	log.info("TOKEN ENABLE FALSE");
        	return;
        }
		
		log.info("TOKEN ENABLE NOT VALID");
		return;
	}
	
}
